package me.wazup.hideandseek;

import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import java.util.Arrays;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataOutputStream;
import java.net.URL;
import javax.net.ssl.HttpsURLConnection;
import java.util.function.Predicate;
import java.util.concurrent.TimeUnit;
import java.nio.charset.StandardCharsets;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;
import java.io.ByteArrayOutputStream;
import java.util.HashSet;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.function.Supplier;
import java.util.concurrent.ScheduledExecutorService;
import java.util.Iterator;
import org.bukkit.World;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.Objects;
import org.bukkit.Bukkit;
import java.io.IOException;
import java.util.UUID;
import org.bukkit.configuration.file.YamlConfiguration;
import java.io.File;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.plugin.Plugin;

public class Metrics
{
    private final Plugin plugin;
    private final MetricsBase metricsBase;
    
    public Metrics(final JavaPlugin javaPlugin, final int n) {
        this.plugin = (Plugin)javaPlugin;
        final File file = new File(new File(javaPlugin.getDataFolder().getParentFile(), "bStats"), "config.yml");
        final YamlConfiguration loadConfiguration = YamlConfiguration.loadConfiguration(file);
        if (!loadConfiguration.isSet("serverUuid")) {
            loadConfiguration.addDefault("enabled", (Object)true);
            loadConfiguration.addDefault("serverUuid", (Object)UUID.randomUUID().toString());
            loadConfiguration.addDefault("logFailedRequests", (Object)false);
            loadConfiguration.addDefault("logSentData", (Object)false);
            loadConfiguration.addDefault("logResponseStatusText", (Object)false);
            loadConfiguration.options().header("bStats (https://bStats.org) collects some basic information for plugin authors, like how\nmany people use their plugin and their total player count. It's recommended to keep bStats\nenabled, but if you're not comfortable with this, you can turn this setting off. There is no\nperformance penalty associated with having metrics enabled, and data sent to bStats is fully\nanonymous.").copyDefaults(true);
            try {
                loadConfiguration.save(file);
            }
            catch (final IOException ex) {}
        }
        final boolean boolean1 = loadConfiguration.getBoolean("enabled", true);
        final String string = loadConfiguration.getString("serverUuid");
        final boolean boolean2 = loadConfiguration.getBoolean("logFailedRequests", false);
        final boolean boolean3 = loadConfiguration.getBoolean("logSentData", false);
        final boolean boolean4 = loadConfiguration.getBoolean("logResponseStatusText", false);
        final String s = "bukkit";
        final String s2 = string;
        final boolean b = boolean1;
        final Consumer<JsonObjectBuilder> consumer = this::appendPlatformData;
        final Consumer<JsonObjectBuilder> consumer2 = this::appendServiceData;
        final Consumer consumer3 = runnable -> Bukkit.getScheduler().runTask((Plugin)javaPlugin2, runnable);
        Objects.requireNonNull(javaPlugin);
        this.metricsBase = new MetricsBase(s, s2, n, b, consumer, consumer2, consumer3, javaPlugin::isEnabled, (msg, thrown) -> this.plugin.getLogger().log(Level.WARNING, msg, thrown), msg2 -> this.plugin.getLogger().log(Level.INFO, msg2), boolean2, boolean3, boolean4);
    }
    
    public void addCustomChart(final CustomChart customChart) {
        this.metricsBase.addCustomChart(customChart);
    }
    
    private void appendPlatformData(final JsonObjectBuilder jsonObjectBuilder) {
        jsonObjectBuilder.appendField("playerAmount", this.getPlayerAmount());
        jsonObjectBuilder.appendField("onlineMode", Bukkit.getOnlineMode() ? 1 : 0);
        jsonObjectBuilder.appendField("bukkitVersion", Bukkit.getVersion());
        jsonObjectBuilder.appendField("bukkitName", Bukkit.getName());
        jsonObjectBuilder.appendField("javaVersion", System.getProperty("java.version"));
        jsonObjectBuilder.appendField("osName", System.getProperty("os.name"));
        jsonObjectBuilder.appendField("osArch", System.getProperty("os.arch"));
        jsonObjectBuilder.appendField("osVersion", System.getProperty("os.version"));
        jsonObjectBuilder.appendField("coreCount", Runtime.getRuntime().availableProcessors());
    }
    
    private void appendServiceData(final JsonObjectBuilder jsonObjectBuilder) {
        jsonObjectBuilder.appendField("pluginVersion", this.plugin.getDescription().getVersion());
    }
    
    private int getPlayerAmount() {
        int n = 0;
        final Iterator iterator = Bukkit.getWorlds().iterator();
        while (iterator.hasNext()) {
            n += ((World)iterator.next()).getPlayers().size();
        }
        return n;
    }
    
    public static class MetricsBase
    {
        public static final String METRICS_VERSION = "2.2.1";
        private static final ScheduledExecutorService scheduler;
        private static final String REPORT_URL = "https://bStats.org/api/v2/data/%s";
        private final String platform;
        private final String serverUuid;
        private final int serviceId;
        private final Consumer<JsonObjectBuilder> appendPlatformDataConsumer;
        private final Consumer<JsonObjectBuilder> appendServiceDataConsumer;
        private final Consumer<Runnable> submitTaskConsumer;
        private final Supplier<Boolean> checkServiceEnabledSupplier;
        private final BiConsumer<String, Throwable> errorLogger;
        private final Consumer<String> infoLogger;
        private final boolean logErrors;
        private final boolean logSentData;
        private final boolean logResponseStatusText;
        private final Set<CustomChart> customCharts;
        private final boolean enabled;
        
        public MetricsBase(final String platform, final String serverUuid, final int serviceId, final boolean enabled, final Consumer<JsonObjectBuilder> appendPlatformDataConsumer, final Consumer<JsonObjectBuilder> appendServiceDataConsumer, final Consumer<Runnable> submitTaskConsumer, final Supplier<Boolean> checkServiceEnabledSupplier, final BiConsumer<String, Throwable> errorLogger, final Consumer<String> infoLogger, final boolean logErrors, final boolean logSentData, final boolean logResponseStatusText) {
            this.customCharts = new HashSet<CustomChart>();
            this.platform = platform;
            this.serverUuid = serverUuid;
            this.serviceId = serviceId;
            this.enabled = enabled;
            this.appendPlatformDataConsumer = appendPlatformDataConsumer;
            this.appendServiceDataConsumer = appendServiceDataConsumer;
            this.submitTaskConsumer = submitTaskConsumer;
            this.checkServiceEnabledSupplier = checkServiceEnabledSupplier;
            this.errorLogger = errorLogger;
            this.infoLogger = infoLogger;
            this.logErrors = logErrors;
            this.logSentData = logSentData;
            this.logResponseStatusText = logResponseStatusText;
            this.checkRelocation();
            if (enabled) {
                this.startSubmitting();
            }
        }
        
        private static byte[] compress(final String s) {
            if (s == null) {
                return null;
            }
            final ByteArrayOutputStream out = new ByteArrayOutputStream();
            try (final GZIPOutputStream gzipOutputStream = new GZIPOutputStream(out)) {
                gzipOutputStream.write(s.getBytes(StandardCharsets.UTF_8));
            }
            return out.toByteArray();
        }
        
        public void addCustomChart(final CustomChart customChart) {
            this.customCharts.add(customChart);
        }
        
        private void startSubmitting() {
            final Runnable runnable = () -> {
                if (!this.enabled || !this.checkServiceEnabledSupplier.get()) {
                    MetricsBase.scheduler.shutdown();
                    return;
                }
                else {
                    if (this.submitTaskConsumer != null) {
                        this.submitTaskConsumer.accept(this::submitData);
                    }
                    else {
                        this.submitData();
                    }
                    return;
                }
            };
            final long n = (long)(60000.0 * (3.0 + Math.random() * 3.0));
            final long n2 = (long)(60000.0 * (Math.random() * 30.0));
            MetricsBase.scheduler.schedule(runnable, n, TimeUnit.MILLISECONDS);
            MetricsBase.scheduler.scheduleAtFixedRate(runnable, n + n2, 1800000L, TimeUnit.MILLISECONDS);
        }
        
        private void submitData() {
            final JsonObjectBuilder jsonObjectBuilder3 = new JsonObjectBuilder();
            this.appendPlatformDataConsumer.accept(jsonObjectBuilder3);
            final JsonObjectBuilder jsonObjectBuilder2 = new JsonObjectBuilder();
            this.appendServiceDataConsumer.accept(jsonObjectBuilder2);
            final JsonObjectBuilder.JsonObject[] array = this.customCharts.stream().map(customChart -> customChart.getRequestJsonObject(this.errorLogger, this.logErrors)).filter(Objects::nonNull).toArray(JsonObjectBuilder.JsonObject[]::new);
            jsonObjectBuilder2.appendField("id", this.serviceId);
            jsonObjectBuilder2.appendField("customCharts", array);
            jsonObjectBuilder3.appendField("service", jsonObjectBuilder2.build());
            jsonObjectBuilder3.appendField("serverUUID", this.serverUuid);
            jsonObjectBuilder3.appendField("metricsVersion", "2.2.1");
            MetricsBase.scheduler.execute(() -> {
                final JsonObjectBuilder jsonObjectBuilder3;
                jsonObjectBuilder3.build();
                try {
                    this.sendData(jsonObject);
                }
                catch (final Exception ex) {
                    if (this.logErrors) {
                        this.errorLogger.accept("Could not submit bStats metrics data", ex);
                    }
                }
            });
        }
        
        private void sendData(final JsonObjectBuilder.JsonObject jsonObject) {
            if (this.logSentData) {
                this.infoLogger.accept("Sent bStats metrics data: " + jsonObject.toString());
            }
            final HttpsURLConnection httpsURLConnection = (HttpsURLConnection)new URL(String.format("https://bStats.org/api/v2/data/%s", this.platform)).openConnection();
            final byte[] compress = compress(jsonObject.toString());
            httpsURLConnection.setRequestMethod("POST");
            httpsURLConnection.addRequestProperty("Accept", "application/json");
            httpsURLConnection.addRequestProperty("Connection", "close");
            httpsURLConnection.addRequestProperty("Content-Encoding", "gzip");
            httpsURLConnection.addRequestProperty("Content-Length", String.valueOf(compress.length));
            httpsURLConnection.setRequestProperty("Content-Type", "application/json");
            httpsURLConnection.setRequestProperty("User-Agent", "Metrics-Service/1");
            httpsURLConnection.setDoOutput(true);
            try (final DataOutputStream dataOutputStream = new DataOutputStream(httpsURLConnection.getOutputStream())) {
                dataOutputStream.write(compress);
            }
            final StringBuilder obj = new StringBuilder();
            try (final BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(httpsURLConnection.getInputStream()))) {
                String line;
                while ((line = bufferedReader.readLine()) != null) {
                    obj.append(line);
                }
            }
            if (this.logResponseStatusText) {
                this.infoLogger.accept("Sent data to bStats and received response: " + (Object)obj);
            }
        }
        
        private void checkRelocation() {
            if (System.getProperty("bstats.relocatecheck") == null || !System.getProperty("bstats.relocatecheck").equals("false")) {
                final String prefix = new String(new byte[] { 111, 114, 103, 46, 98, 115, 116, 97, 116, 115 });
                final String prefix2 = new String(new byte[] { 121, 111, 117, 114, 46, 112, 97, 99, 107, 97, 103, 101 });
                if (MetricsBase.class.getPackage().getName().startsWith(prefix) || MetricsBase.class.getPackage().getName().startsWith(prefix2)) {
                    throw new IllegalStateException("bStats Metrics class has not been relocated correctly!");
                }
            }
        }
        
        static {
            scheduler = Executors.newScheduledThreadPool(1, task -> new Thread(task, "bStats-Metrics"));
        }
    }
    
    public abstract static class CustomChart
    {
        private final String chartId;
        
        protected CustomChart(final String chartId) {
            if (chartId == null) {
                throw new IllegalArgumentException("chartId must not be null");
            }
            this.chartId = chartId;
        }
        
        public JsonObjectBuilder.JsonObject getRequestJsonObject(final BiConsumer<String, Throwable> biConsumer, final boolean b) {
            final JsonObjectBuilder jsonObjectBuilder = new JsonObjectBuilder();
            jsonObjectBuilder.appendField("chartId", this.chartId);
            try {
                final JsonObjectBuilder.JsonObject chartData = this.getChartData();
                if (chartData == null) {
                    return null;
                }
                jsonObjectBuilder.appendField("data", chartData);
            }
            catch (final Throwable t) {
                if (b) {
                    biConsumer.accept("Failed to get data for custom chart with id " + this.chartId, t);
                }
                return null;
            }
            return jsonObjectBuilder.build();
        }
        
        protected abstract JsonObjectBuilder.JsonObject getChartData();
    }
    
    public static class SimplePie extends CustomChart
    {
        private final Callable<String> callable;
        
        public SimplePie(final String s, final Callable<String> callable) {
            super(s);
            this.callable = callable;
        }
        
        @Override
        protected JsonObjectBuilder.JsonObject getChartData() {
            final String s = this.callable.call();
            if (s == null || s.isEmpty()) {
                return null;
            }
            return new JsonObjectBuilder().appendField("value", s).build();
        }
    }
    
    public static class JsonObjectBuilder
    {
        private StringBuilder builder;
        private boolean hasAtLeastOneField;
        
        public JsonObjectBuilder() {
            this.builder = new StringBuilder();
            this.hasAtLeastOneField = false;
            this.builder.append("{");
        }
        
        private static String escape(final String s) {
            final StringBuilder sb = new StringBuilder();
            for (int i = 0; i < s.length(); ++i) {
                final char char1 = s.charAt(i);
                if (char1 == '\"') {
                    sb.append("\\\"");
                }
                else if (char1 == '\\') {
                    sb.append("\\\\");
                }
                else if (char1 <= '\u000f') {
                    sb.append("\\u000").append(Integer.toHexString(char1));
                }
                else if (char1 <= '\u001f') {
                    sb.append("\\u00").append(Integer.toHexString(char1));
                }
                else {
                    sb.append(char1);
                }
            }
            return sb.toString();
        }
        
        public JsonObjectBuilder appendField(final String s, final String s2) {
            if (s2 == null) {
                throw new IllegalArgumentException("JSON value must not be null");
            }
            this.appendFieldUnescaped(s, "\"" + escape(s2) + "\"");
            return this;
        }
        
        public JsonObjectBuilder appendField(final String s, final int i) {
            this.appendFieldUnescaped(s, String.valueOf(i));
            return this;
        }
        
        public JsonObjectBuilder appendField(final String s, final JsonObject jsonObject) {
            if (jsonObject == null) {
                throw new IllegalArgumentException("JSON object must not be null");
            }
            this.appendFieldUnescaped(s, jsonObject.toString());
            return this;
        }
        
        public JsonObjectBuilder appendField(final String s, final JsonObject[] array) {
            if (array == null) {
                throw new IllegalArgumentException("JSON values must not be null");
            }
            this.appendFieldUnescaped(s, "[" + Arrays.stream(array).map((Function<? super JsonObject, ?>)JsonObject::toString).collect((Collector<? super Object, ?, String>)Collectors.joining(",")) + "]");
            return this;
        }
        
        private void appendFieldUnescaped(final String s, final String str) {
            if (this.builder == null) {
                throw new IllegalStateException("JSON has already been built");
            }
            if (s == null) {
                throw new IllegalArgumentException("JSON key must not be null");
            }
            if (this.hasAtLeastOneField) {
                this.builder.append(",");
            }
            this.builder.append("\"").append(escape(s)).append("\":").append(str);
            this.hasAtLeastOneField = true;
        }
        
        public JsonObject build() {
            if (this.builder == null) {
                throw new IllegalStateException("JSON has already been built");
            }
            final JsonObject jsonObject = new JsonObject(this.builder.append("}").toString());
            this.builder = null;
            return jsonObject;
        }
        
        public static class JsonObject
        {
            private final String value;
            
            private JsonObject(final String value) {
                this.value = value;
            }
            
            @Override
            public String toString() {
                return this.value;
            }
        }
    }
}
